require 'aws-sdk-dynamodb'

# Created using https://docs.aws.amazon.com/amazondynamodb/latest/developerguide/GettingStarted.Ruby.01.html
def create_table(dynamodb_client, table_definition)
  response = dynamodb_client.create_table(table_definition)
  response.table_description.table_status
rescue StandardError => e
  puts "Error creating table: #{e.message}"
  'Error'
end

def configure_aws_and_tables

  # Replace us-west-2 with the AWS Region you're using for Amazon DynamoDB.
  # To use the downloadable version of Amazon DynamoDB,
  # uncomment the endpoint statement.
  Aws.config.update(
    endpoint: Application[:settings].dynamodb_url,
    region: Application[:settings].dynamodb_region
  )

  dynamodb_client = Aws::DynamoDB::Client.new
  # https://docs.aws.amazon.com/sdk-for-ruby/v3/api/Aws/DynamoDB/Client.html
  table_definition = {
    "attribute_definitions": [
      {
        "attribute_name": "content_id",
        "attribute_type": "S"
      },
      {
        "attribute_name": "customer_id",
        "attribute_type": "S"
      },
      {
        "attribute_name": "organization_id",
        "attribute_type": "S"
      }
    ],
    "table_name": Application[:settings].dynamodb_table_name,
    "key_schema": [
      {
        "attribute_name": "content_id",
        "key_type": "HASH"
      },
      {
        "attribute_name": "organization_id",
        "key_type": "RANGE"
      }
    ],
    "global_secondary_indexes": [
      {
        "index_name": "flowdown_items",
        "key_schema": [
          {
            "attribute_name": "customer_id",
            "key_type": "HASH"
          },
          {
            "attribute_name": "organization_id",
            "key_type": "RANGE"
          }
        ],
        "projection": {
          "projection_type": "ALL"
        },
        "provisioned_throughput": {
          "read_capacity_units": 1,
          "write_capacity_units": 1
        }
      }
    ],
    "provisioned_throughput": {
      "read_capacity_units": 1,
      "write_capacity_units": 1
    },
    "stream_specification": {
      "stream_enabled": false
    }
  }

  create_table_result = create_table(dynamodb_client, table_definition)

  if create_table_result == 'Error'
    puts "Failed to create schema, please check logs for more details"
  else
    puts "Schema created with status '#{create_table_result}'."
  end
end

configure_aws_and_tables if $PROGRAM_NAME == __FILE__
